<?php

namespace Drupal\beehotel_pricealterator\Form;

use Drupal\beehotel_pricealterator\PriceAlteratorPluginManager;
use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Database\Connection;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\Routing\RouteProviderInterface;
use Drupal\Core\Link;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Price Alterators draggable list form.
 *
 * @ingroup beehotel_pricealterator
 */
class AlteratorsList extends FormBase {

  /**
   * Drupal configuration service container.
   *
   * @var \Drupal\Core\Config\ConfigFactory
   */
  protected $configFactory;

  /**
   * The route provider.
   *
   * @var \Drupal\Core\Routing\RouteProviderInterface
   */
  protected $routeProvider;

  /**
   * The plugin manager.
   *
   * @var \Drupal\beehotel_pricealterator\PriceAlteratorPluginManager
   */
  protected $priceAlteratorPluginManager;

  /**
   * The renderer service.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected $renderer;

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * {@inheritdoc}
   *
   * @param \Drupal\Core\Config\ConfigFactory $config_factory
   *   The config factory.
   * @param \Drupal\Core\Routing\RouteProviderInterface $provider
   *   The route provider.
   * @param \Drupal\beehotel_pricealterator\PriceAlteratorPluginManager $price_alterator_plugin_manager
   *   The tempstore factory.
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   The renderer service.
   * @param \Drupal\Core\Database\Connection $database
   *   The database connection.
   */
  public function __construct(ConfigFactory $config_factory, RouteProviderInterface $provider, PriceAlteratorPluginManager $price_alterator_plugin_manager, RendererInterface $renderer, Connection $database) {
    $this->configFactory = $config_factory;
    $this->routeProvider = $provider;
    $this->priceAlteratorPluginManager = $price_alterator_plugin_manager;
    $this->renderer = $renderer;
    $this->database = $database;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('router.route_provider'),
      $container->get('plugin.manager.beehotel.pricealterator'),
      $container->get('renderer'),
      $container->get('database'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'alteratorslist';
  }

  /**
   * Builds the simple tabledrag form.
   *
   * @param array $form
   *   Render array representing from.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   Current form state.
   *
   * @return array
   *   The render array defining the elements of the form.
   */
  public function buildForm(array $form, FormStateInterface $form_state) {

    $what_is_this  = t('This is the <b>draggable price alterator List</b> to create you custom price algorithm.');
    $what_is_this .= t('A very powelful tool to boost your income and define your business stategy');
    $what_is_this .= t("Here's what you can:");
    $what_is_this .= '<ul>';
    $what_is_this .= '<li>' . t('<b>Sort</b>: Price alterators are processed following order defined in this table. This setting has strong impact of your price algorithm. Change order carefully. Click save at the end of page to confirm the order.');
    $what_is_this .= '<li>' . t('<b>Enable</b>: Every price alterator can be enabled at will. Click on settings to set up.');
    $what_is_this .= '<li>' . t('<b>Quote</b>: Every price alterator can  be quoted at will. Click on settings to set up.');
    $what_is_this .= '</ul>';

    $what_is_this .= "The price Alterator system is flexible and expandible, to build  your own price algorithm at wish. More alterators can be added to your business:";
    $what_is_this .= '<ul>';
    $what_is_this .= '<li>' . t("Read <a href='https://www.drupal.org/node/3354393' target='_blank'>documentation</a>");
    $what_is_this .= '<li>' . t("Ask the coder for a <a href='https://www.drupal.org/user/50680' target='_blank'>quote</a>");
    $what_is_this .= '</ul>';

    $form['whatisthis'] = [
      '#type' => 'details',
      '#title' => t('Price Alterators, How to'),
      '#description' => $what_is_this,
      '#open' => FALSE,
    ];

    $form['alterators'] = [
      '#type' => 'table',

      '#header' => [
        'title' => $this->t('Name'),
        'description' => $this->t('Description'),
        'current_value' => $this->t('Current value'),
        'settings' => $this->t('Settings'),
        'enabled' => $this->t('Enabled'),
        'weight' => $this->t('Weight'),
      ],

      '#empty' => $this->t('Sorry, There are no items!'),
      '#tabledrag' => [
        [
          'action' => 'order',
          'relationship' => 'sibling',
          'group' => 'table-sort-weight',
        ],
      ],
    ];

    $data['alterators'] = $this->priceAlteratorPluginManager->alterators([]);
    $data['values'] = \Drupal::config('beehotel_pricealterator.settings');
    $rows = [];

    foreach ($data['alterators'] as $id => $values) {

      $tmp = [];
      $values = $values;
      $tmp['route'] = BEEHOTEL_PRICEALTERATOR_ROUTE_BASE . strtolower($id) . '_settings';
      $tmp['exists'] = count($this->routeProvider->getRoutesByNames([$tmp['route']]));

      if ($tmp['exists'] == 1) {
        $tmp['url'] = Url::fromRoute(BEEHOTEL_PRICEALTERATOR_ROUTE_BASE . strtolower($id) . '_settings');
        $tmp['settings'] = Link::fromTextAndUrl($this->t("Settings"), $tmp['url'])->toString();
      }

      $tmp['config'] = $this->configFactory
        ->getEditable(BEEHOTEL_PRICEALTERATOR_ROUTE_BASE . strtolower($id) . '.settings');

      $tmp['mandatory'] = ($values['type'] == 'mandatory') ? '<span class="mandatory">*</span>' : '';

      $form['alterators'][$values['id']]['#attributes']['class'][] = 'draggable';
      $form['alterators'][$values['id']]['#attributes']['class'][] = strtolower($values['id']);
      $form['alterators'][$values['id']]['#weight'] = $values['get_user_weight'];

      $form['alterators'][$values['id']]['name'] = [
        '#type' => 'html_tag',
        '#tag' => 'span',
        '#value' => $values['id'],
        '#attributes' => ['class' => ['name', strtolower($values['id'])]],
      ];

      $form['alterators'][$values['id']]['description'] = [
        '#markup' => $values['description'],
      ];

      $form['alterators'][$values['id']]['current_value'] = [
        '#attributes' => [
          'class' => [
            'current-value', strtolower($values['id']),
          ],
        ],
        '#tag' => 'p',
        '#type' => 'html_tag',
        '#value' => $values['current_value'],
      ];

      if (!isset($tmp['settings'])) {
        $tmp['settings'] = "---";
      }

      $form['alterators'][$values['id']]['settings'] = [
        '#type' => 'html_tag',
        '#tag' => 'p',
        '#value' => $tmp['settings'],
      ];

      $form['alterators'][$values['id']]['enabled'] = [
        '#default_value' => $values['enabled'],
        '#type' => 'html_tag',
        '#tag' => 'span',
        '#attributes' => [
          'class' => ['enabled', 'enabled-' . $values['enabled']],
        ],
      ];

      $form['alterators'][$values['id']]['weight'] = [
        '#type' => 'weight',
        '#title' => $this->t('Weight for @title', ['@title' => $values['id']]),
        '#title_display' => 'invisible',
        '#default_value' => $values['get_user_weight'],
        '#attributes' => ['class' => ['table-sort-weight']],
      ];
    }

    $form['actions'] = ['#type' => 'actions'];
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save Sorting'),
    ];
    $form['actions']['cancel'] = [
      '#type' => 'submit',
      '#value'  => 'Cancel',
      '#attributes' => [
        'title' => $this->t('Reset Sorting'),
      ],
      '#submit' => ['::cancel'],
      '#limit_validation_errors' => [],
    ];

    $form['#attached']['library'][] = 'beehotel_pricealterator/pricealterators';

    return $form;
  }

  /**
   * Form submission handler for the 'Return to' action.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   */
  public function cancel(array &$form, FormStateInterface $form_state) {
    $form_state->setRedirect('beehotel_pricealterator.info.chain');
  }

  /**
   * Form submission handler for the simple form.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $alterators = [];
    foreach ($form_state->getValue('alterators') as $key => $value) {
      $alterators[$key . "_weight"] = $value['weight'];
    }

    \Drupal::service('config.factory')
      ->getEditable('beehotel_pricealterator.settings')
      ->set('price_alterators', $alterators)
      ->save();
  }

}
